Type: Package
Package: genieBPC
Title: Project GENIE BioPharma Collaborative Data Processing Pipeline
Version: 2.2.0
Authors@R: 
    c(
    person(given = "Jessica A.",
             family = "Lavery",
             role = c("aut", "cre"),
             email = "laveryj@mskcc.org",
             comment = c(ORCID = "0000-0002-2746-5647")),
      person(given = "Michael A.",
             family = "Curry",
             role = "aut",
             comment = c(ORCID = "0000-0002-0261-4044")),
      person(given = "Samantha",
             family = "Brown",
             role = "aut",
             comment = c(ORCID = "0000-0001-5352-974X")),
      person(given = "Karissa",
             family = "Whiting",
             role = "aut",
             comment = c(ORCID = "0000-0002-4683-1868")),
      person(given = "Yufei",
             family = "Deng",
             role = "aut",
             comment = c(ORCID = "0009-0000-3352-1126")),
      person(given = "Hannah",
             family = "Fuchs",
             role = "ctb",
             comment = c(ORCID = "0000-0003-2426-0096")),
      person(given = "Axel",
             family = "Martin",
             role = "ctb"),
      person(given = "Daniel D.",
             family = "Sjoberg",
             role = c("ctb"),
             email = "danield.sjoberg@gmail.com",
             comment = c(ORCID = "0000-0003-0862-2018")))
Description: The American Association Research (AACR) Project
    Genomics Evidence Neoplasia Information Exchange (GENIE) BioPharma
    Collaborative represents a multi-year, multi-institution effort
    to build a pan-cancer repository of linked clinico-genomic data.
    The genomic and clinical data are provided in multiple releases (separate
    releases for each cancer cohort with updates following data corrections),
    which are stored on the data sharing platform
    'Synapse' <https://www.synapse.org/>.
    The 'genieBPC' package provides a seamless way to obtain the
    data corresponding to each release from 'Synapse' and to prepare
    datasets for analysis.
License: MIT + file LICENSE
BugReports: https://github.com/GENIE-BPC/genieBPC/issues
Depends: R (>= 3.5)
Imports: cli (>= 2.5.0), dplyr (>= 1.0.7), dtplyr (>= 1.1.0), httr,
        jsonlite, purrr (>= 1.0.2), rlang (>= 1.0.0), stringr (>=
        1.4.0), sunburstR, tibble (>= 3.1.2), tidyr
Suggests: covr (>= 3.5.1), ggplot2 (>= 3.3.5), gt (>= 0.3.0), gtsummary
        (>= 1.5.2), knitr (>= 1.33), magrittr (>= 2.0.1), plotly (>=
        4.10.0), rmarkdown (>= 2.8), testthat (>= 3.0.0), markdown,
        spelling
VignetteBuilder: rmarkdown, knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: TRUE
RoxygenNote: 7.3.3
URL: https://genie-bpc.github.io/genieBPC/
Language: en-US
NeedsCompilation: no
Packaged: 2026-04-20 13:29:28 UTC; laveryj
Author: Jessica A. Lavery [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2746-5647>),
  Michael A. Curry [aut] (ORCID: <https://orcid.org/0000-0002-0261-4044>),
  Samantha Brown [aut] (ORCID: <https://orcid.org/0000-0001-5352-974X>),
  Karissa Whiting [aut] (ORCID: <https://orcid.org/0000-0002-4683-1868>),
  Yufei Deng [aut] (ORCID: <https://orcid.org/0009-0000-3352-1126>),
  Hannah Fuchs [ctb] (ORCID: <https://orcid.org/0000-0003-2426-0096>),
  Axel Martin [ctb],
  Daniel D. Sjoberg [ctb] (ORCID:
    <https://orcid.org/0000-0003-0862-2018>)
Maintainer: Jessica A. Lavery <laveryj@mskcc.org>
Repository: CRAN
Date/Publication: 2026-04-20 20:50:11 UTC
Built: R 4.6.0; ; 2026-04-25 13:42:25 UTC; unix
